@@ -6,7 +6,8 @@ Created on Mon Sep 16 00:02:57 2019
 """
 import pandas as pd
 from thermosteam.utils import colors
-from biosteam.evaluation.evaluation_tools import plot_spearman
+from biosteam.plots import plot_spearman
+from biorefineries.cornstover.model import cornstover_model
 
 # %%  Plot Spearman correlations
 
@@ -22,26 +23,27 @@ replacement_labels = {
     'Saccharification and co fermentation-R301 saccharification conversion': 'Saccharification conversion',
     'Saccharification and co fermentation-R301 ethanol conversion': 'Fermentation ethanol conversion',
     'Boiler turbogenerator-BT boiler efficiency': 'Boiler efficiency',
-    'Boiler turbogenerator boiler base cost': 'Boiler base cost',
-    'Boiler turbogenerator turbogenerator base cost': 'Turbogenerator base cost',
-    'Pretreatment reactor system base cost': 'Pretreatment reactor base cost',
+    'Boiler turbogenerator boiler base cost': 'Boiler baseline cost',
+    'Boiler turbogenerator turbogenerator base cost': 'Turbogenerator baseline cost',
+    'Pretreatment reactor system base cost': 'Pretreatment reactor baseline cost',
     'Power utility price': 'Electricity price',
-    'Cooling tower base cost': 'Cooling tower base cost',
-    'Waste water system cost waste water system base cost': 'Wastewater treatment base cost',
+    'Cooling tower base cost': 'Cooling tower baseline cost',
+    'Waste water system cost waste water system base cost': 'Wastewater treatment baseline cost',
     'Waste water system cost waste water system exponent': 'Wastewater treatment exponent'}
 
 
-def replace_label_text(label_text):
+def var_to_label(var):
     """Replace label text for graph."""
-    name, distribution = label_text.split(' [')
-    lb, mid, ub = eval('[' + distribution)
+    description = var.describe()
+    name, distribution = description.split(' (')
+    lb, mid, ub = eval('(' + distribution)
     if 'efficiency' in name:
         distribution = f" ({lb:.2f}, {mid:.2f}, {ub:.2f})"
     else:
         distribution = f" ({lb:.3g}, {mid:.3g}, {ub:.3g})"
-    pos = name.find(' (')
+    pos = name.find(' [')
     if pos != -1:
-        units = str(name[pos:]).replace('(', '[').replace(')', ']')
+        units = str(name[pos:])
         if units == ' [USD/kg]':
             units = ' [$\mathrm{USD} \cdot \mathrm{kg}^{-1}$]'
         elif units == ' [USD/kWhr]':
@@ -55,15 +57,19 @@ def replace_label_text(label_text):
         name = replacement_labels[name]
     return name + units + distribution
 
-# Get data
-rhos = pd.read_excel('Spearman correlation cornstover.xlsx',
-                     header=[0], index_col=0).iloc[:, 0]
+# # Get data
+rhos = pd.read_excel('results/Spearman correlation cornstover.xlsx',
+                      header=[0], index_col=[0, 1]).iloc[:, 0]
 
 # Get only important parameters
-rhos = rhos[rhos.abs()>0.055] 
+rhos = rhos[rhos.abs() > 0.055] 
+labels_by_index = {i.index: var_to_label(i) for i in cornstover_model.get_parameters() 
+                   if i.index in rhos.index}
+labels = [labels_by_index[i] for i in rhos.index]
+rhos = pd.Series(data=rhos.values, index=labels)
 
-# Plot and fix axis labels
+# # Plot and fix axis labels
 fig, ax = plot_spearman(rhos, top=10, name='MESP')
-labels = [item.get_text() for item in ax.get_yticklabels()]
-new_labels = [replace_label_text(i) for i in labels]
-ax.set_yticklabels(new_labels)
\ No newline at end of file
+# labels = [item.get_text() for item in ax.get_yticklabels()]
+# new_labels = [replace_label_text(i) for i in labels]
+# ax.set_yticklabels(new_labels)
\ No newline at end of file
