@@ -20,10 +20,10 @@ __all__ = (
     'test_lipidcane',
     'test_cornstover',
     'test_LAOs',
-    'test_wheatstraw',
-    'test_animal_bedding',
     'test_lactic',
     'test_ethanol_adipic',
+    'test_wheatstraw',
+    'test_animal_bedding',
     'generate_all_code',
     'generate_code',
     'print_results',
@@ -41,9 +41,9 @@ feedstocks_by_module = {
     'LAOs': 'glucose',
     'lactic': 'feedstock',
     'ethanol_adipic': 'feedstock',
-    'HP': 'feedstock',
     'wheatstraw': 'wheatstraw',
     'animal_bedding': 'bedding',
+    # 'HP': 'feedstock',
 }
 products_by_module = {
     'corn': 'ethanol',
@@ -144,6 +144,28 @@ def print_results(tea):
     print('Electricity consumption:', units.get_electricity_consumption())
     print('Electricity production:', units.get_electricity_production())
     
+
+    bst.process_tools.default()
+    from biorefineries import corn as module
+    try: module.load()
+    except: pass
+    feedstock = module.corn
+    product = module.ethanol
+    tea = module.corn_tea
+    units = UnitGroup('Biorefinery', tea.units)
+    assert np.allclose(tea.IRR, 0.037843194745526644, rtol=1e-2)
+    assert np.allclose(feedstock.price, 0.13227735731092652, rtol=1e-2)
+    assert np.allclose(product.price, 0.48547915353569393, rtol=1e-2)
+    assert np.allclose(tea.sales, 73911402.04639979, rtol=1e-2)
+    assert np.allclose(tea.material_cost, 54229609.23083669, rtol=1e-2)
+    assert np.allclose(tea.installed_equipment_cost, 67931892.15209496, rtol=1e-2)
+    assert np.allclose(tea.utility_cost, 8267419.421774661, rtol=1e-2)
+    assert np.allclose(units.get_heating_duty(), 135.60034920381048, rtol=1e-2)
+    assert np.allclose(units.get_cooling_duty(), 141.00001509562642, rtol=1e-2)
+    assert np.allclose(units.get_electricity_consumption(), 2.340125651483439, rtol=1e-2)
+    assert np.allclose(units.get_electricity_production(), 0.0, rtol=1e-2)
+    bst.process_tools.default()
+    
 def test_corn():
     bst.process_tools.default()
     from biorefineries import corn as module
@@ -153,16 +175,16 @@ def test_corn():
     product = module.ethanol
     tea = module.corn_tea
     units = UnitGroup('Biorefinery', tea.units)
-    assert np.allclose(tea.IRR, 0.06359351306940697, rtol=1e-2)
+    assert np.allclose(tea.IRR, 0.037843194745526644, rtol=1e-2)
     assert np.allclose(feedstock.price, 0.13227735731092652, rtol=1e-2)
     assert np.allclose(product.price, 0.48547915353569393, rtol=1e-2)
-    assert np.allclose(tea.sales, 73911903.87893353, rtol=1e-2)
-    assert np.allclose(tea.material_cost, 54229612.27968673, rtol=1e-2)
-    assert np.allclose(tea.installed_equipment_cost, 67888346.46815833, rtol=1e-2)
-    assert np.allclose(tea.utility_cost, 6646814.904073888, rtol=1e-2)
-    assert np.allclose(units.get_heating_duty(), 93.75005124179637, rtol=1e-2)
-    assert np.allclose(units.get_cooling_duty(), 96.87211085978647, rtol=1e-2)
-    assert np.allclose(units.get_electricity_consumption(), 2.3389275658240734, rtol=1e-2)
+    assert np.allclose(tea.sales, 73911402.04639979, rtol=1e-2)
+    assert np.allclose(tea.material_cost, 54229609.23083669, rtol=1e-2)
+    assert np.allclose(tea.installed_equipment_cost, 67931892.15209496, rtol=1e-2)
+    assert np.allclose(tea.utility_cost, 8267419.421774661, rtol=1e-2)
+    assert np.allclose(units.get_heating_duty(), 135.60034920381048, rtol=1e-2)
+    assert np.allclose(units.get_cooling_duty(), 141.00001509562642, rtol=1e-2)
+    assert np.allclose(units.get_electricity_consumption(), 2.340125651483439, rtol=1e-2)
     assert np.allclose(units.get_electricity_production(), 0.0, rtol=1e-2)
     bst.process_tools.default()
     
@@ -175,17 +197,17 @@ def test_lipidcane():
     product = module.ethanol
     tea = module.lipidcane_tea
     units = UnitGroup('Biorefinery', tea.units)
-    assert np.allclose(tea.IRR, 0.18049812456891415, rtol=1e-2)
+    assert np.allclose(tea.IRR, 0.2004155048202737, rtol=1e-2)
     assert np.allclose(feedstock.price, 0.03455, rtol=1e-2)
     assert np.allclose(product.price, 0.789, rtol=1e-2)
-    assert np.allclose(tea.sales, 102493171.42530274, rtol=1e-2)
-    assert np.allclose(tea.material_cost, 61819200.17615199, rtol=1e-2)
-    assert np.allclose(tea.installed_equipment_cost, 202305311.36327285, rtol=1e-2)
-    assert np.allclose(tea.utility_cost, -21065582.11058501, rtol=1e-2)
-    assert np.allclose(units.get_heating_duty(), 276.94917232588114, rtol=1e-2)
-    assert np.allclose(units.get_cooling_duty(), 245.98384752626484, rtol=1e-2)
-    assert np.allclose(units.get_electricity_consumption(), 9.581267484994973, rtol=1e-2)
-    assert np.allclose(units.get_electricity_production(), 77.09915886507521, rtol=1e-2)
+    assert np.allclose(tea.sales, 108312511.2255254, rtol=1e-2)
+    assert np.allclose(tea.material_cost, 61686342.60634536, rtol=1e-2)
+    assert np.allclose(tea.installed_equipment_cost, 210882564.3909881, rtol=1e-2)
+    assert np.allclose(tea.utility_cost, -24773469.818018492, rtol=1e-2)
+    assert np.allclose(units.get_heating_duty(), 256.62596387100984, rtol=1e-2)
+    assert np.allclose(units.get_cooling_duty(), 250.58558038981556, rtol=1e-2)
+    assert np.allclose(units.get_electricity_consumption(), 9.431177030531131, rtol=1e-2)
+    assert np.allclose(units.get_electricity_production(), 88.83332388315456, rtol=1e-2)
     bst.process_tools.default()
     
 def test_cornstover():
@@ -199,15 +221,15 @@ def test_cornstover():
     units = UnitGroup('Biorefinery', tea.units)
     assert np.allclose(tea.IRR, 0.1, rtol=1e-2)
     assert np.allclose(feedstock.price, 0.05158816935126135, rtol=1e-2)
-    assert np.allclose(product.price, 0.7130236349343888, rtol=1e-2)
-    assert np.allclose(tea.sales, 134171543.31470321, rtol=1e-2)
-    assert np.allclose(tea.material_cost, 80366427.42525397, rtol=1e-2)
-    assert np.allclose(tea.installed_equipment_cost, 218206214.34651473, rtol=1e-2)
-    assert np.allclose(tea.utility_cost, -8369523.722273722, rtol=1e-2)
-    assert np.allclose(units.get_heating_duty(), 341.6190758403897, rtol=1e-2)
-    assert np.allclose(units.get_cooling_duty(), 384.3630438280598, rtol=1e-2)
-    assert np.allclose(units.get_electricity_consumption(), 23.854606165361538, rtol=1e-2)
-    assert np.allclose(units.get_electricity_production(), 41.25381002701907, rtol=1e-2)
+    assert np.allclose(product.price, 0.7326263699008247, rtol=1e-2)
+    assert np.allclose(tea.sales, 133807745.01444514, rtol=1e-2)
+    assert np.allclose(tea.material_cost, 85260005.53515732, rtol=1e-2)
+    assert np.allclose(tea.installed_equipment_cost, 208166358.26559696, rtol=1e-2)
+    assert np.allclose(tea.utility_cost, -11186247.570852375, rtol=1e-2)
+    assert np.allclose(units.get_heating_duty(), 308.68234969096864, rtol=1e-2)
+    assert np.allclose(units.get_cooling_duty(), 353.46195670628, rtol=1e-2)
+    assert np.allclose(units.get_electricity_consumption(), 23.248856504335308, rtol=1e-2)
+    assert np.allclose(units.get_electricity_production(), 46.50368060074836, rtol=1e-2)
     bst.process_tools.default()
     
 def test_sugarcane():
@@ -219,17 +241,17 @@ def test_sugarcane():
     product = module.ethanol
     tea = module.sugarcane_tea
     units = UnitGroup('Biorefinery', tea.units)
-    assert np.allclose(tea.IRR, 0.10748625239122699, rtol=1e-2)
+    assert np.allclose(tea.IRR, 0.12022422663689111, rtol=1e-2)
     assert np.allclose(feedstock.price, 0.03455, rtol=1e-2)
     assert np.allclose(product.price, 0.789, rtol=1e-2)
-    assert np.allclose(tea.sales, 87955077.0538273, rtol=1e-2)
-    assert np.allclose(tea.material_cost, 60192842.50809233, rtol=1e-2)
-    assert np.allclose(tea.installed_equipment_cost, 162936659.34374115, rtol=1e-2)
-    assert np.allclose(tea.utility_cost, -6446162.4286760185, rtol=1e-2)
-    assert np.allclose(units.get_heating_duty(), 312.98253582785566, rtol=1e-2)
-    assert np.allclose(units.get_cooling_duty(), 250.2056638160375, rtol=1e-2)
-    assert np.allclose(units.get_electricity_consumption(), 9.630688343934322, rtol=1e-2)
-    assert np.allclose(units.get_electricity_production(), 30.291465358921567, rtol=1e-2)
+    assert np.allclose(tea.sales, 87280075.39839575, rtol=1e-2)
+    assert np.allclose(tea.material_cost, 59826991.53802018, rtol=1e-2)
+    assert np.allclose(tea.installed_equipment_cost, 208869333.83743256, rtol=1e-2)
+    assert np.allclose(tea.utility_cost, -19090287.761531264, rtol=1e-2)
+    assert np.allclose(units.get_heating_duty(), 289.8228204324192, rtol=1e-2)
+    assert np.allclose(units.get_cooling_duty(), 278.6429484448828, rtol=1e-2)
+    assert np.allclose(units.get_electricity_consumption(), 15.63950095751762, rtol=1e-2)
+    assert np.allclose(units.get_electricity_production(), 76.15723272336585, rtol=1e-2)
     bst.process_tools.default()
     
 def test_LAOs():
@@ -243,103 +265,50 @@ def test_LAOs():
     units = UnitGroup('Biorefinery', tea.units)
     assert np.allclose(tea.IRR, 0.1, rtol=1e-2)
     assert np.allclose(feedstock.price, 0.265, rtol=1e-2)
-    assert np.allclose(product.price, 1.3044340148279439, rtol=1e-2)
-    assert np.allclose(tea.sales, 163064302.30503127, rtol=1e-2)
-    assert np.allclose(tea.material_cost, 135661590.95377028, rtol=1e-2)
-    assert np.allclose(tea.installed_equipment_cost, 61273897.855782405, rtol=1e-2)
-    assert np.allclose(tea.utility_cost, 2790455.9147288934, rtol=1e-2)
-    assert np.allclose(units.get_heating_duty(), 40.03596321617748, rtol=1e-2)
-    assert np.allclose(units.get_cooling_duty(), 125.33344794500373, rtol=1e-2)
-    assert np.allclose(units.get_electricity_consumption(), 3.596749741993362, rtol=1e-2)
-    assert np.allclose(units.get_electricity_production(), 3.5967497419933645, rtol=1e-2)
+    assert np.allclose(product.price, 1.325979028649637, rtol=1e-2)
+    assert np.allclose(tea.sales, 161219012.55369785, rtol=1e-2)
+    assert np.allclose(tea.material_cost, 135661164.88724265, rtol=1e-2)
+    assert np.allclose(tea.installed_equipment_cost, 57006040.1082998, rtol=1e-2)
+    assert np.allclose(tea.utility_cost, 1554844.4698447797, rtol=1e-2)
+    assert np.allclose(units.get_heating_duty(), 39.587331392856626, rtol=1e-2)
+    assert np.allclose(units.get_cooling_duty(), 127.39469106251018, rtol=1e-2)
+    assert np.allclose(units.get_electricity_consumption(), 3.5306112844050603, rtol=1e-2)
+    assert np.allclose(units.get_electricity_production(), 7.271296144805024, rtol=1e-2)
     bst.process_tools.default()
     
 def test_lactic():
     bst.process_tools.default()
     from biorefineries import lactic as module
-    try: module.load()
-    except: pass
+    module.load_system('SSCF')
     feedstock = module.feedstock
     product = module.lactic_acid
     tea = module.lactic_tea
     units = UnitGroup('Biorefinery', tea.units)
+    module.lactic_sys.simulate()
     assert np.allclose(tea.IRR, 0.1, rtol=1e-2)
     assert np.allclose(feedstock.price, 0.06287583717512708, rtol=1e-2)
-    assert np.allclose(product.price, 1.5271616857518349, rtol=1e-2)
-    assert np.allclose(tea.sales, 331936305.3904605, rtol=1e-2)
-    assert np.allclose(tea.material_cost, 219074528.09864402, rtol=1e-2)
-    assert np.allclose(tea.installed_equipment_cost, 319908749.01807934, rtol=1e-2)
-    assert np.allclose(tea.utility_cost, 25050313.992293335, rtol=1e-2)
-    assert np.allclose(units.get_heating_duty(), 1829.09043477846, rtol=1e-2)
-    assert np.allclose(units.get_cooling_duty(), 1895.6439958358947, rtol=1e-2)
-    assert np.allclose(units.get_electricity_consumption(), 45.390871189920496, rtol=1e-2)
+    assert np.allclose(tea.solve_price(product), 1.237548148885074, rtol=1e-2)
+    assert np.allclose(tea.sales, 54143215.78374311, rtol=1e-2)
+    assert np.allclose(tea.material_cost, 219104068.95373908, rtol=1e-2)
+    assert np.allclose(tea.installed_equipment_cost, 320073822.05670184, rtol=1e-2)
+    assert np.allclose(tea.utility_cost, 22275814.89016047, rtol=1e-2)
+    assert np.allclose(units.get_heating_duty(), 1827.5926437830065, rtol=1e-2)
+    assert np.allclose(units.get_cooling_duty(), 1895.6520852552394, rtol=1e-2)
+    assert np.allclose(units.get_electricity_consumption(), 40.363511796333356, rtol=1e-2)
     assert np.allclose(units.get_electricity_production(), 0.0, rtol=1e-2)
     bst.process_tools.default()
     
+# Work is ongoing, at this stage, it is fine as long as these modules can load
+# and systems can be simulated
 def test_ethanol_adipic():
     bst.process_tools.default()
     from biorefineries import ethanol_adipic as module
-    try: module.load()
-    except: pass
-    feedstock = module.feedstock
-    product = module.ethanol
-    tea = module.ethanol_adipic_tea
-    units = UnitGroup('Biorefinery', tea.units)
-    assert np.allclose(tea.IRR, 0.1, rtol=1e-2)
-    assert np.allclose(feedstock.price, 0.06287581915485817, rtol=1e-2)
-    assert np.allclose(product.price, 0.8352254290016348, rtol=1e-2)
-    assert np.allclose(tea.sales, 219751260.2006, rtol=1e-2)
-    assert np.allclose(tea.material_cost, 120581699.4153729, rtol=1e-2)
-    assert np.allclose(tea.installed_equipment_cost, 282909147.0811014, rtol=1e-2)
-    assert np.allclose(tea.utility_cost, 14242922.86112985, rtol=1e-2)
-    assert np.allclose(units.get_heating_duty(), 296.834875396348, rtol=1e-2)
-    assert np.allclose(units.get_cooling_duty(), 247.78343626334382, rtol=1e-2)
-    assert np.allclose(units.get_electricity_consumption(), 26.56706595101369, rtol=1e-2)
-    assert np.allclose(units.get_electricity_production(), 0.0, rtol=1e-2)
-    bst.process_tools.default()
-
-def test_HP_cellulosic():
-    bst.process_tools.default()
-    from biorefineries import HP as module
-    try: module.load('cellulosic')
-    except: pass
-    feedstock = module.feedstock
-    product = module.AcrylicAcid
-    tea = module.HP_tea
-    units = UnitGroup('Biorefinery', tea.units)
-    assert np.allclose(tea.IRR, 0.1, rtol=1e-2)
-    assert np.allclose(feedstock.price, 0.0628405632131775, rtol=1e-2)
-    assert np.allclose(product.price, 1.311604006683178, rtol=1e-2)
-    assert np.allclose(tea.sales, 283742631.8995883, rtol=1e-2)
-    assert np.allclose(tea.material_cost, 162124736.8600144, rtol=1e-2)
-    assert np.allclose(tea.installed_equipment_cost, 375378915.8994427, rtol=1e-2)
-    assert np.allclose(tea.utility_cost, 14130636.879378846, rtol=1e-2)
-    assert np.allclose(units.get_heating_duty(), 538.7730297007987, rtol=1e-2)
-    assert np.allclose(units.get_cooling_duty(), 673.8593732991667, rtol=1e-2)
-    assert np.allclose(units.get_electricity_consumption(), 25.604546059612314, rtol=1e-2)
-    assert np.allclose(units.get_electricity_production(), 0.0, rtol=1e-2)
-    bst.process_tools.default()
-    
-def test_HP_sugarcane():
-    bst.process_tools.default()
-    from biorefineries import HP as module
-    try: module.load('sugarcane')
-    except: pass
-    feedstock = module.feedstock
-    product = module.AcrylicAcid
-    tea = module.HP_tea
-    units = UnitGroup('Biorefinery', tea.units)
-    assert np.allclose(tea.IRR, 0.1, rtol=1e-2)
-    assert np.allclose(feedstock.price, 0.03455, rtol=1e-2)
-    assert np.allclose(product.price, 1.088310160650574, rtol=1e-2)
-    assert np.allclose(tea.sales, 254169785.51489013, rtol=1e-2)
-    assert np.allclose(tea.material_cost, 140314059.9280608, rtol=1e-2)
-    assert np.allclose(tea.installed_equipment_cost, 348397129.1412859, rtol=1e-2)
-    assert np.allclose(tea.utility_cost, 9896574.99874393, rtol=1e-2)
-    assert np.allclose(units.get_heating_duty(), 677.6593151558291, rtol=1e-2)
-    assert np.allclose(units.get_cooling_duty(), 598.6323442594456, rtol=1e-2)
-    assert np.allclose(units.get_electricity_consumption(), 17.932476260679728, rtol=1e-2)
-    assert np.allclose(units.get_electricity_production(), 0.0, rtol=1e-2)
+    module.load_system('acid', 'HMPP')
+    module.biorefinery.simulate()
+    module.load_system('AFEX', 'CPP_AFEX')
+    module.biorefinery.simulate()
+    module.load_system('base', 'HMPP')
+    module.biorefinery.simulate()
     bst.process_tools.default()
     
 @pytest.mark.slow
@@ -354,15 +323,15 @@ def test_wheatstraw():
     units = UnitGroup('Biorefinery', tea.units)
     assert np.allclose(tea.IRR, 0.1, rtol=1e-2)
     assert np.allclose(feedstock.price, 0.056999999999999995, rtol=1e-2)
-    assert np.allclose(product.price, 0.9091269068657728, rtol=1e-2)
-    assert np.allclose(tea.sales, 128110733.17694265, rtol=1e-2)
-    assert np.allclose(tea.material_cost, 63430743.5116114, rtol=1e-2)
-    assert np.allclose(tea.installed_equipment_cost, 236439520.41561243, rtol=1e-2)
-    assert np.allclose(tea.utility_cost, -5352589.6271707155, rtol=1e-2)
-    assert np.allclose(units.get_heating_duty(), 214.26465184757853, rtol=1e-2)
-    assert np.allclose(units.get_cooling_duty(), 261.6818186290857, rtol=1e-2)
-    assert np.allclose(units.get_electricity_consumption(), 22.96620132771446, rtol=1e-2)
-    assert np.allclose(units.get_electricity_production(), 34.093572633573686, rtol=1e-2)
+    assert np.allclose(product.price, 0.9033321094343063, rtol=1e-2)
+    assert np.allclose(tea.sales, 128751220.49360518, rtol=1e-2)
+    assert np.allclose(tea.material_cost, 63431132.07841703, rtol=1e-2)
+    assert np.allclose(tea.installed_equipment_cost, 237441843.41517347, rtol=1e-2)
+    assert np.allclose(tea.utility_cost, -5034648.66311628, rtol=1e-2)
+    assert np.allclose(units.get_heating_duty(), 216.35813625516792, rtol=1e-2)
+    assert np.allclose(units.get_cooling_duty(), 265.7465556328792, rtol=1e-2)
+    assert np.allclose(units.get_electricity_consumption(), 23.125593758589847, rtol=1e-2)
+    assert np.allclose(units.get_electricity_production(), 33.592005154050234, rtol=1e-2)
     bst.process_tools.default()
     
 @pytest.mark.slow
@@ -377,17 +346,18 @@ def test_animal_bedding():
     units = UnitGroup('Biorefinery', tea.units)
     assert np.allclose(tea.IRR, 0.1, rtol=1e-2)
     assert np.allclose(feedstock.price, 0.003, rtol=1e-2)
-    assert np.allclose(product.price, 0.784263871897465, rtol=1e-2)
-    assert np.allclose(tea.sales, 112892087.24258891, rtol=1e-2)
-    assert np.allclose(tea.material_cost, 33147675.58326452, rtol=1e-2)
-    assert np.allclose(tea.installed_equipment_cost, 278921960.54721147, rtol=1e-2)
-    assert np.allclose(tea.utility_cost, -2034797.6940362304, rtol=1e-2)
-    assert np.allclose(units.get_heating_duty(), 157.88540205395574, rtol=1e-2)
-    assert np.allclose(units.get_cooling_duty(), 198.2373645805114, rtol=1e-2)
-    assert np.allclose(units.get_electricity_consumption(), 32.050076008170485, rtol=1e-2)
-    assert np.allclose(units.get_electricity_production(), 36.280168552331276, rtol=1e-2)
+    assert np.allclose(product.price, 0.7890118048567955, rtol=1e-2)
+    assert np.allclose(tea.sales, 107275778.85699502, rtol=1e-2)
+    assert np.allclose(tea.material_cost, 32460366.631773323, rtol=1e-2)
+    assert np.allclose(tea.installed_equipment_cost, 262912439.97063047, rtol=1e-2)
+    assert np.allclose(tea.utility_cost, -2560757.2115471847, rtol=1e-2)
+    assert np.allclose(units.get_heating_duty(), 165.1624907961539, rtol=1e-2)
+    assert np.allclose(units.get_cooling_duty(), 211.85054691475685, rtol=1e-2)
+    assert np.allclose(units.get_electricity_consumption(), 29.19232139864, rtol=1e-2)
+    assert np.allclose(units.get_electricity_production(), 34.515818677863315, rtol=1e-2)
     bst.process_tools.default()
     
+    
 
 ### DO NOT DELETE:
 ### Code commented for legacy purposes
@@ -441,16 +411,16 @@ def test_animal_bedding():
 #     assert np.allclose(units.get_electricity_consumption(), 26.565278642577344, rtol=0.001)
 #     assert np.allclose(units.get_electricity_production(), 0.0)
     
-if __name__ == '__main__':
-    test_corn()
-    test_sugarcane()
-    test_lipidcane()
-    test_cornstover()
-    test_HP_cellulosic()
-    test_HP_sugarcane()
-    test_LAOs()
-    test_lactic()
-    test_ethanol_adipic()
+# if __name__ == '__main__':
+    # test_corn()
+    # test_sugarcane()
+    # test_lipidcane()
+    # test_cornstover()
+    # # test_HP_cellulosic()
+    # # test_HP_sugarcane()
+    # test_LAOs()
+    # test_lactic()
+    # test_ethanol_adipic()
 
 
 
